@@ -1,79 +1,27 @@
 package Mojo::Pg::Che::Database;
 
-use Mojo::Base 'Mojo::EventEmitter'; #'Mojo::Pg::Database';
+#~ use Mojo::Base 'Mojo::EventEmitter';
+use Mojo::Base 'Mojo::Pg::Database';
 use Carp qw(croak shortmess);
 use DBD::Pg ':async';
-use Mojo::IOLoop;
+#~ use Mojo::IOLoop;
+
 use Mojo::Pg::Che::Results;
 use Mojo::Pg::Transaction;
-#~ use Scalar::Util 'weaken';
+
 
 my $handler_err = sub {$_[0] = shortmess $_[0]; 0;};
 has handler_err => sub {$handler_err};
-
-has [qw(dbh)];
-has pg => undef, weak => 1;
-
 has results_class => 'Mojo::Pg::Che::Results';
+has debug => sub { shift->pg->debug };
 
 my $PKG = __PACKAGE__;
 
-sub disconnect {#  copy/paste Mojo::Pg::Database
-  my $self = shift;
-  $self->_unwatch;
-  $self->dbh->disconnect;
-}
-
-sub is_listening { !!keys %{shift->{listen} || {}} }#  copy/paste Mojo::Pg::Database
-
-sub listen {#  copy/paste Mojo::Pg::Database
-  my ($self, $name) = @_;
-
-  my $dbh = $self->dbh;
-  $dbh->do('listen ' . $dbh->quote_identifier($name))
-    unless $self->{listen}{$name}++;
-  $self->_watch;
-
-  return $self;
-}
-
-sub unlisten {#  copy/paste Mojo::Pg::Database
-  my ($self, $name) = @_;
-
-  my $dbh = $self->dbh;
-  $dbh->do('unlisten ' . $dbh->quote_identifier($name));
-  $name eq '*' ? delete $self->{listen} : delete $self->{listen}{$name};
-  $self->_unwatch unless $self->{waiting} || $self->is_listening;
-
-  return $self;
-}
-
-sub _notifications {#  copy/paste Mojo::Pg::Database
-  my $self = shift;
-  my $dbh  = $self->dbh;
-  while (my $n = $dbh->pg_notifies) { $self->emit(notification => @$n) }
-}
-
-sub notify {#  copy/paste Mojo::Pg::Database
-  my ($self, $name, $payload) = @_;
-
-  my $dbh    = $self->dbh;
-  my $notify = 'notify ' . $dbh->quote_identifier($name);
-  $notify .= ', ' . $dbh->quote($payload) if defined $payload;
-  $dbh->do($notify);
-  $self->_notifications;
-
-  return $self;
-}
-
-sub pid { shift->dbh->{pg_pid} } #  copy/paste Mojo::Pg::Database
-
-sub ping { shift->dbh->ping } #  copy/paste Mojo::Pg::Database
-
 sub query { shift->select(@_) }
 
 sub execute_sth {
   my ($self, $sth,) = map shift, 1..2;
+  #~ warn "execute_sth: ", $self->dbh;
   
   my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
   
@@ -86,12 +34,12 @@ sub execute_sth {
   local $sth->{HandleError} = $self->handler_err;
   
   eval {$sth->execute(@_)}#binds
-    or die "Bad statement: ", $@, $sth->{Statement};
+    or die "Bad statement: ", $@;#, $sth->{Statement};
   
   # Blocking
   unless ($cb) {#
     $self->_notifications;
-    return $self->results_class->new(sth => $sth);
+    return $self->results_class->new(db => $self, sth => $sth);
   }
   
   # Non-blocking
@@ -115,28 +63,33 @@ sub prepare {
   
   my $dbh = $self->dbh;
   
-  $attrs->{pg_async} = PG_ASYNC
-    if delete $attrs->{Async};
-  
-  return $dbh->prepare_cached($query, $attrs, 3)
-    if delete $attrs->{Cached};
-  
-  return $dbh->prepare($query, $attrs);
+  #~ $attrs->{pg_async} = PG_ASYNC
+    #~ if delete $attrs->{Async};
+
+  my $sth = delete $attrs->{Cached}
+    ? $dbh->prepare_cached($query, $attrs, 3)
+    : $dbh->prepare($query, $attrs);
   
+  #~ $sth->{private_mojo_db} = $self;
+  return $sth;
 }
 
-sub prepare_cached { shift->dbh->prepare_cached(@_); }
+sub prepare_cached { 
+  my $self = shift;
+  
+  my $sth = $self->dbh->prepare_cached(@_);
+  #~ $sth->{private_mojo_db} = $self;
+  return $sth;
+}
 
-sub tx {shift->begin}
+sub tx { shift->begin }
 sub begin {
   my $self = shift;
   return $self->{tx}
     if $self->{tx};
   
   my $tx = $self->{tx} = Mojo::Pg::Transaction->new(db => $self);
-  #~ weaken $tx->{db};
   return $tx;
-
 }
 
 sub commit {
@@ -148,10 +101,7 @@ sub commit {
 
 sub rollback {
   my $self = shift;
-  my $tx = delete $self->{tx}
-    or return;
-  $tx = undef;# DESTROY
-  
+  delete $self->{tx};# DESTROY
 }
 
 my @DBH_METHODS = qw(
@@ -197,15 +147,15 @@ sub _DBH_METHOD {
   my $cb = ref $_[-1] eq 'CODE' ? pop : undef;
   
   $attrs->{pg_async} = PG_ASYNC
-    if $cb || delete $attrs->{Async};
+    if $cb;## || delete $attrs->{Async};
   
   $sth->{pg_async} = PG_ASYNC
     if $sth && $attrs->{pg_async};
 
   $sth ||= $self->prepare($query, $attrs);
   
-  $cb ||= $self->_async_cb()
-    if $attrs->{pg_async};
+  #~ $cb ||= $self->_async_cb()
+    #~ if $attrs->{pg_async};
   
   my @bind = @_;
   
@@ -220,6 +170,27 @@ sub _DBH_METHOD {
   
 }
 
+
+sub DESTROY {#  copy/paste Mojo::Pg::Database + rollback
+  my $self = shift;
+  
+  $self->rollback;
+  
+  my $waiting = $self->{waiting};
+  $waiting->{cb}($self, 'Premature connection close', undef) if $waiting->{cb};
+ 
+  return unless (my $pg = $self->pg) && (my $dbh = $self->dbh);
+  $pg->_enqueue($dbh)
+    #~ and ($self->debug && say STDERR "DESTROY $dbh")
+    unless $dbh->{private_mojo_no_reuse};
+
+}
+
+
+1;
+
+__END__
+
 sub _async_cb {
   my $self = shift;
   my ($result, $err);
@@ -231,7 +202,6 @@ sub _async_cb {
   };
 }
 
-
 sub _watch {
   my $self = shift;
 
@@ -256,42 +226,10 @@ sub _watch {
       my $result = do { local $dbh->{RaiseError} = 0; $dbh->pg_result };
       my $err = defined $result ? undef : $dbh->errstr;
 
-      eval { $self->$cb($err, $self->results_class->new(sth => $sth)); };
-      #~ warn "Non-blocking callback result error: ", $@
-      #~ $reactor->{cb_error} = $@
-        #~ if $@;
-      
+      eval { $self->$cb($err, $self->results_class->new(db => $self, sth => $sth)); };
       $self->_unwatch unless $self->{waiting} || $self->is_listening;
     }
   )->watch($self->{handle}, 1, 0);
   
   return \$cb, \$sth;
 }
-
-sub _unwatch {#  copy/paste Mojo::Pg::Database
-  my $self = shift;
-  return unless delete $self->{watching};
-  Mojo::IOLoop->singleton->reactor->remove($self->{handle});
-  $self->emit('close') if $self->is_listening;
-}
-
-sub DESTROY {#  copy/paste Mojo::Pg::Database + rollback
-  my $self = shift;
-  
-  $self->rollback;
-  
-  my $waiting = $self->{waiting};
-  if (my $cb = $waiting->{cb}) {
-    $self->$cb('Premature connection close', undef)
-      if ref $cb eq 'CODE';
-    
-  }
-  #~ $waiting->{cb}($self, 'Premature connection close', undef) ;
-
-  return unless (my $pg = $self->pg) && (my $dbh = $self->dbh);
-  $pg->_enqueue($dbh);
-  
-  #~ $self->SUPER::DESTROY;
-}
-
-1;
\ No newline at end of file
